;;; test-helper.el --- shut-up: Initialize test suite  -*- lexical-binding: t; -*-

;; Copyright (C) 2014, 2015  Sebastian Wiesner <swiesner@lunaryorn.com>

;; This file is NOT part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;; Initialize test suite for `ert-runner'.

;;; Code:

(require 's)
(require 'f)

;; Load shut-up from the source directory
(let ((source-directory (locate-dominating-file (f-this-file) "Cask")))
  (load (expand-file-name "shut-up" source-directory)))

;;; test-helper.el ends here
